@@ -18,8 +18,9 @@
 
 /* remote host account manipulation (POP/IMAP) */
 
-#include "account.h"
 #include "mutt.h"
+#include "account.h"
+#include "url.h"
 
 /* mutt_account_match: compare account info (host/port/user) */
 int mutt_account_match (const ACCOUNT* a1, const ACCOUNT* a2)
@@ -53,9 +54,39 @@ int mutt_account_match (const ACCOUNT* a1, const ACCOUNT* a2)
   return 1;
 }
 
+/* mutt_account_fromurl: fill account with information from url. */
+int mutt_account_fromurl (ACCOUNT* account, ciss_url_t* url)
+{
+  /* must be present */
+  if (url->host)
+    strfcpy (account->host, url->host, sizeof (account->host));
+  else
+    return -1;
+
+  if (url->user)
+  {
+    strfcpy (account->user, url->user, sizeof (account->user));
+    account->flags |= M_ACCT_USER;
+  }
+  if (url->pass)
+  {
+    strfcpy (account->pass, url->pass, sizeof (account->pass));
+    account->flags |= M_ACCT_PASS;
+  }
+  if (url->port)
+  {
+    account->port = url->port;
+    account->flags |= M_ACCT_PORT;
+  }
+
+  return 0;
+}
+
 /* mutt_account_getuser: retrieve username into ACCOUNT, if neccessary */
 int mutt_account_getuser (ACCOUNT* account)
 {
+  char prompt[SHORT_STRING];
+
   /* already set */
   if (account->flags & M_ACCT_USER)
     return 0;
@@ -70,9 +101,9 @@ int mutt_account_getuser (ACCOUNT* account)
   /* prompt (defaults to unix username), copy into account->user */
   else
   {
+    snprintf (prompt, sizeof (prompt), _("Username at %s: "), account->host);
     strfcpy (account->user, NONULL (Username), sizeof (account->user));
-    if (mutt_get_field (_("Username: "), account->user,
-        sizeof (account->user), 0))
+    if (mutt_get_field (prompt, account->user, sizeof (account->user), 0))
       return -1;
   }
 
@@ -84,6 +115,8 @@ int mutt_account_getuser (ACCOUNT* account)
 /* mutt_account_getpass: fetch password into ACCOUNT, if neccessary */
 int mutt_account_getpass (ACCOUNT* account)
 {
+  char prompt[SHORT_STRING];
+
   if (account->flags & M_ACCT_PASS)
     return 0;
 #ifdef USE_IMAP
@@ -96,9 +129,10 @@ int mutt_account_getpass (ACCOUNT* account)
 #endif
   else
   {
+    snprintf (prompt, sizeof (prompt), _("Password for %s@%s: "),
+      account->user, account->host);
     account->pass[0] = '\0';
-    if (mutt_get_field (_("Password: "), account->pass,
-        sizeof (account->pass), M_PASS))
+    if (mutt_get_field (prompt, account->pass, sizeof (account->pass), M_PASS))
       return -1;
   }
 
