@@ -144,6 +144,7 @@ folder_format_str (char *dest, size_t destlen, char op, const char *src,
   FOLDER *folder = (FOLDER *) data;
   struct passwd *pw;
   struct group *gr;
+  int optional = (flags & M_FORMAT_OPTIONAL);
 
   switch (op)
   {
@@ -158,14 +159,10 @@ folder_format_str (char *dest, size_t destlen, char op, const char *src,
 	tnow = time (NULL);
 	t_fmt = tnow - folder->ff->st->st_mtime < 31536000 ? "%b %d %H:%M" : "%b %d  %Y";
 	strftime (date, sizeof (date), t_fmt, localtime (&folder->ff->st->st_mtime));
-	snprintf (tmp, sizeof (tmp), "%%%ss", fmt);
-	snprintf (dest, destlen, tmp, date);
+	mutt_format_s (dest, destlen, fmt, date);
       }
       else
-      {
-	snprintf (tmp, sizeof (tmp), "%%%ss", fmt);
-	snprintf (dest, destlen, tmp, "");
-      }
+	mutt_format_s (dest, destlen, fmt, "");
       break;
       
     case 'f':
@@ -181,8 +178,7 @@ folder_format_str (char *dest, size_t destlen, char op, const char *src,
 		(S_ISDIR (folder->ff->st->st_mode) ? "/" : 
 		 ((folder->ff->st->st_mode & S_IXUSR) != 0 ? "*" : "")));
       }
-      snprintf (tmp, sizeof (tmp), "%%%ss", fmt);
-      snprintf (dest, destlen, tmp, fn);
+      mutt_format_s (dest, destlen, fmt, fn);
       break;
       
     case 'F':
@@ -199,32 +195,26 @@ folder_format_str (char *dest, size_t destlen, char op, const char *src,
 	    (folder->ff->st->st_mode & S_IROTH) != 0 ? 'r' : '-',
 	    (folder->ff->st->st_mode & S_IWOTH) != 0 ? 'w' : '-',
 	    (folder->ff->st->st_mode & S_ISVTX) != 0 ? 't' : (folder->ff->st->st_mode & S_IXOTH) != 0 ? 'x': '-');
-	snprintf (tmp, sizeof (tmp), "%%%ss", fmt);
-	snprintf (dest, destlen, tmp, permission);
+	mutt_format_s (dest, destlen, fmt, permission);
       }
-      else
-      {
 #ifdef USE_IMAP
-	if (folder->ff->imap)
-	{
-	  /* mark folders with subfolders AND mail */
-	  sprintf (permission, "IMAP %c",
-            (folder->ff->inferiors && folder->ff->selectable) ? '+' : ' ');
-          snprintf (tmp, sizeof (tmp), "%%%ss", fmt);
-          snprintf (dest, destlen, tmp, permission);
-	}                                        
+      else if (folder->ff->imap)
+      {
+	/* mark folders with subfolders AND mail */
+	sprintf (permission, "IMAP %c",
+          (folder->ff->inferiors && folder->ff->selectable) ? '+' : ' ');
+	mutt_format_s (dest, destlen, fmt, permission);
+      }                                        
 #endif
-      }
+      else
+	mutt_format_s (dest, destlen, fmt, "");
       break;
       
     case 'g':
       if (folder->ff->st != NULL)
       {
 	if ((gr = getgrgid (folder->ff->st->st_gid)))
-	{
-	  snprintf (tmp, sizeof (tmp), "%%%ss", fmt);
-	  snprintf (dest, destlen, tmp, gr->gr_name);
-	}
+	  mutt_format_s (dest, destlen, fmt, gr->gr_name);
 	else
 	{
 	  snprintf (tmp, sizeof (tmp), "%%%sld", fmt);
@@ -232,10 +222,7 @@ folder_format_str (char *dest, size_t destlen, char op, const char *src,
 	}
       }
       else
-      {
-	snprintf (tmp, sizeof (tmp), "%%%ss", fmt);
-	snprintf (dest, destlen, tmp, "");
-      }
+	mutt_format_s (dest, destlen, fmt, "");
       break;
       
     case 'l':
@@ -245,15 +232,25 @@ folder_format_str (char *dest, size_t destlen, char op, const char *src,
 	snprintf (dest, destlen, tmp, folder->ff->st->st_nlink);
       }
       else
-      {
-	snprintf (tmp, sizeof (tmp), "%%%ss", fmt);
-	snprintf (dest, destlen, tmp, "");
-      }
+	mutt_format_s (dest, destlen, fmt, "");
       break;
       
     case 'N':
+#ifdef USE_IMAP
+      if (mx_is_imap (folder->ff->desc))
+      {
+	if (!optional)
+	{
+	  snprintf (tmp, sizeof (tmp), "%%%sd", fmt);
+	  snprintf (dest, destlen, tmp, folder->ff->new);
+	}
+	else if (!folder->ff->new)
+	  optional = 0;
+	break;
+      }
+#endif
       snprintf (tmp, sizeof (tmp), "%%%sc", fmt);
-      snprintf (dest, destlen, tmp, folder->ff->is_new ? 'N' : ' ');
+      snprintf (dest, destlen, tmp, folder->ff->new ? 'N' : ' ');
       break;
       
     case 's':
@@ -263,10 +260,7 @@ folder_format_str (char *dest, size_t destlen, char op, const char *src,
 	snprintf (dest, destlen, tmp, (long) folder->ff->st->st_size);
       }
       else
-      {
-	snprintf (tmp, sizeof (tmp), "%%%ss", fmt);
-	snprintf (dest, destlen, tmp, "");
-      }
+	mutt_format_s (dest, destlen, fmt, "");
       break;
       
     case 't':
@@ -278,10 +272,7 @@ folder_format_str (char *dest, size_t destlen, char op, const char *src,
       if (folder->ff->st != NULL)
       {
 	if ((pw = getpwuid (folder->ff->st->st_uid)))
-	{
-	  snprintf (tmp, sizeof (tmp), "%%%ss", fmt);
-	  snprintf (dest, destlen, tmp, pw->pw_name);
-	}
+	  mutt_format_s (dest, destlen, fmt, pw->pw_name);
 	else
 	{
 	  snprintf (tmp, sizeof (tmp), "%%%sld", fmt);
@@ -289,10 +280,7 @@ folder_format_str (char *dest, size_t destlen, char op, const char *src,
 	}
       }
       else
-      {
-	snprintf (tmp, sizeof (tmp), "%%%ss", fmt);
-	snprintf (dest, destlen, tmp, "");
-      }
+	mutt_format_s (dest, destlen, fmt, "");
       break;
       
     default:
@@ -300,6 +288,12 @@ folder_format_str (char *dest, size_t destlen, char op, const char *src,
       snprintf (dest, destlen, tmp, op);
       break;
   }
+
+  if (optional)
+    mutt_FormatString (dest, destlen, ifstring, folder_format_str, data, 0);
+  else if (flags & M_FORMAT_OPTIONAL)
+    mutt_FormatString (dest, destlen, elsestring, folder_format_str, data, 0);
+
   return (src);
 }
 
@@ -327,7 +321,7 @@ static void add_folder (MUTTMENU *m, struct browser_state *state,
     memcpy ((state->entry)[state->entrylen].st, s, sizeof (struct stat));
   }
 
-  (state->entry)[state->entrylen].is_new = new;
+  (state->entry)[state->entrylen].new = new;
   (state->entry)[state->entrylen].name = safe_strdup (name);
   (state->entry)[state->entrylen].desc = safe_strdup (name);
 #ifdef USE_IMAP
@@ -815,8 +809,24 @@ void _mutt_select_file (char *f, size_t flen, int buffy,
 	mutt_ungetch (0, OP_CHECK_NEW);
 	break;
 
-      case OP_NEW_MAILBOX:
-	mutt_error (_("Creating mailboxes is not yet supported."));
+      case OP_CREATE_MAILBOX:
+	if (!state.imap_browse)
+	  mutt_error (_("Create is only supported for IMAP mailboxes"));
+	else
+	{
+	  imap_mailbox_create (LastDir);
+	  /* TODO: find a way to detect if the new folder would appear in
+	   *   this window, and insert it without starting over. */
+	  destroy_state (&state);
+	  init_state (&state, NULL);
+	  state.imap_browse = 1;
+	  imap_browse (LastDir, &state);
+	  menu->data = state.entry;
+	  menu->current = 0; 
+	  menu->top = 0; 
+	  init_menu (&state, menu, title, sizeof (title), buffy);
+	  MAYBE_REDRAW (menu->redraw);
+	}
 	break;
 
       case OP_DELETE_MAILBOX:
@@ -824,7 +834,7 @@ void _mutt_select_file (char *f, size_t flen, int buffy,
 	  mutt_error (_("Delete is only supported for IMAP mailboxes"));
 	else
         {
-	  char msg[LONG_STRING];
+	  char msg[SHORT_STRING];
 	  IMAP_MBOX mx;
 	  int nentry = menu->current;
 	  
@@ -850,6 +860,7 @@ void _mutt_select_file (char *f, size_t flen, int buffy,
 	  }
 	  else
 	    mutt_message _("Mailbox not deleted.");
+	  FREE (&mx.mbox);
         }
         break;
 #endif
