@@ -230,23 +230,11 @@ int mutt_buffy_check (int force)
   char path[_POSIX_PATH_MAX];
   struct stat contex_sb;
   time_t t;
+
 #ifdef USE_IMAP
-  static time_t last_imap_check = 0;
-  int do_imap_check = 1;
-
-  if (ImapCheckTimeout)
-  {
-    time_t now = time (NULL);
-    if (now - last_imap_check < ImapCheckTimeout)
-      do_imap_check = 0;
-    else
-      last_imap_check = now;
-  }
-
   /* update postponed count as well, on force */
   if (force)
     mutt_update_num_postponed ();
-
 #endif
 
   /* fastest return if there are no mailboxes */
@@ -272,37 +260,37 @@ int mutt_buffy_check (int force)
   
   for (tmp = Incoming; tmp; tmp = tmp->next)
   {
+    tmp->new = 0;
+
 #ifdef USE_IMAP
     if ((tmp->magic == M_IMAP) || mx_is_imap (tmp->path))
       tmp->magic = M_IMAP;
     else
 #endif
+    if (stat (tmp->path, &sb) != 0 || sb.st_size == 0 ||
+	(!tmp->magic && (tmp->magic = mx_get_magic (tmp->path)) <= 0))
     {
-      tmp->new = 0;
-      
-      if (stat (tmp->path, &sb) != 0 || sb.st_size == 0 ||
-	  (!tmp->magic && (tmp->magic = mx_get_magic (tmp->path)) <= 0))
-      {
-	/* if the mailbox still doesn't exist, set the newly created flag to
-	 * be ready for when it does.
-	 */
-	tmp->newly_created = 1;
-	tmp->magic = 0;
+      /* if the mailbox still doesn't exist, set the newly created flag to
+       * be ready for when it does. */
+      tmp->newly_created = 1;
+      tmp->magic = 0;
 #ifdef BUFFY_SIZE
-	tmp->size = 0;
+      tmp->size = 0;
 #endif
-	continue;
-      }
+      continue;
     }
 
     /* check to see if the folder is the currently selected folder
      * before polling */
     if (!Context || !Context->path || 
 #ifdef USE_IMAP
-        /* unless folder is an IMAP folder */
-        tmp->magic == M_IMAP ||
+	(tmp->magic == M_IMAP && mutt_strcmp (tmp->path, Context->path)) ||
+	(tmp->magic != M_IMAP && (
 #endif
 	sb.st_dev != contex_sb.st_dev || sb.st_ino != contex_sb.st_ino)
+#ifdef USE_IMAP
+	  ))
+#endif
     {
       switch (tmp->magic)
       {
@@ -355,22 +343,10 @@ int mutt_buffy_check (int force)
 
 #ifdef USE_IMAP
       case M_IMAP:
-        /* poll on do_imap_check, else return cached value.
-         * If the check is forced (eg on mailbox open), check only current
-         * folder */
-        if (do_imap_check || (force && Context && Context->path &&
-          !mutt_strcmp (Context->path, tmp->path)))
-        {
-          tmp->new = 0;
-          if (imap_mailbox_check (tmp->path, 1) > 0)
-          {
-            BuffyCount++;
-            tmp->new = 1;
-          }
-        }
-        else
-          if (tmp->new)
-            BuffyCount++;
+	if ((tmp->new = imap_mailbox_check (tmp->path, 1)) > 0)
+	  BuffyCount++;
+	else
+	  tmp->new = 0;
 
 	break;
 #endif
