@@ -31,6 +31,11 @@
 #include <errno.h>
 #include <ctype.h>
 
+#ifdef HAVE_LANGINFO_YESEXPR
+#include <langinfo.h>
+#include <regex.h>
+#endif
+
 /* not possible to unget more than one char under some curses libs, and it
  * is impossible to unget function keys in SLang, so roll our own input
  * buffering routines.
@@ -142,38 +147,51 @@ void mutt_edit_file (const char *editor, const char *data)
 int mutt_yesorno (const char *msg, int def)
 {
   event_t ch;
-  unsigned char *yes = (unsigned char *) _("yes");
-  unsigned char *no = (unsigned char *) _("no");
-  char yes1 = 'y';
-  char no1 = 'n';
+  char *yes = _("yes");
+  char *no = _("no");
 
-  /*
-   * The keys are not localised, because none of the other
-   * keys in mutt are localised. Also, non-ASCII characters
-   * are unlikely to work at present ...
-   */
+#ifdef HAVE_LANGINFO_YESEXPR
+  regex_t reyes;
+  regex_t reno;
+  int reyes_ok = ! regcomp (& reyes, nl_langinfo (YESEXPR), REG_NOSUB);
+  int reno_ok = ! regcomp (& reno, nl_langinfo (NOEXPR), REG_NOSUB);
+  char answer[2];
+
+  answer[1] = 0;
+#endif
 
   CLEARLINE(LINES-1);
-  if (*yes == yes1 && *no == no1) /* English, or not localised */
-    printw ("%s ([%c]/%c): ", msg, def ? yes1 : no1,
-	    def ? no1 : yes1);
-  else
-    printw ("%s ([%c=%s]/%c=%s): ", msg,
-	    def ? yes1 : no1, def ? yes : no,
-	    def ? no1 : yes1, def ? no : yes);
+  printw ("%s ([%s]/%s): ", msg, def ? yes : no, def ? no : yes);
   FOREVER
   {
     mutt_refresh ();
     ch = mutt_getch ();
-    if (ch.ch == -1) return(-1);
     if (CI_is_return (ch.ch))
       break;
-    else if (tolower (ch.ch) == tolower (yes1))
+    if (ch.ch == -1)
+    {
+      def = -1;
+      break;
+    }
+
+#ifdef HAVE_LANGINFO_YESEXPR
+    answer[0] = ch.ch;
+    if (reyes_ok ? 
+	(regexec (& reyes, answer, 0, 0, 0) == 0) :
+#else
+    if (
+#endif
+	(tolower (ch.ch) == 'y'))
     {
       def = 1;
       break;
     }
-    else if (tolower (ch.ch) == tolower (no1))
+    else if (
+#ifdef HAVE_LANGINFO_YESEXPR
+	     reno_ok ?
+	     (regexec (& reno, answer, 0, 0, 0) == 0) :
+#endif
+	     (tolower (ch.ch) == 'n'))
     {
       def = 0;
       break;
@@ -183,8 +201,19 @@ int mutt_yesorno (const char *msg, int def)
       BEEP();
     }
   }
-  addstr ((char *) (def ? yes : no));
-  mutt_refresh ();
+
+#ifdef HAVE_LANGINFO_YESEXPR    
+  if (reyes_ok)
+    regfree (& reyes);
+  if (reno_ok)
+    regfree (& reno);
+#endif
+
+  if (def >= 0)
+  {
+    addstr ((char *) (def ? yes : no));
+    mutt_refresh ();
+  }
   return (def);
 }
 
@@ -198,7 +227,7 @@ void mutt_query_exit (void)
   if (mutt_yesorno (_("Exit Mutt?"), 1) == 1)
   {
     endwin ();
-    exit (0);
+    exit (1);
   }
   mutt_clear_error();
   mutt_curs_set (-1);
@@ -540,6 +569,39 @@ void mutt_format_string (char *dest, size_t destlen,
   }
 }
 
+/*
+ * This formats a string rather like
+ *   snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
+ *   snprintf (dest, destlen, fmt, s);
+ * except that the numbers in the conversion specification refer to
+ * the number of character cells when printed.
+ */
+
+void mutt_format_s (char *dest,
+		    size_t destlen,
+		    const char *prefix,
+		    const char *s)
+{
+  int right_justify = 1;
+  char *p;
+  int min_width;
+  int max_width = INT_MAX;
+
+  if (*prefix == '-')
+    ++prefix, right_justify = 0;
+  min_width = strtol (prefix, &p, 10);
+  if (*p == '.')
+  {
+    prefix = p + 1;
+    max_width = strtol (prefix, &p, 10);
+    if (p <= prefix)
+      max_width = INT_MAX;
+  }
+
+  mutt_format_string (dest, destlen, min_width, max_width,
+		      right_justify, ' ', s, mutt_strlen (s));
+}
+
 /*
  * mutt_paddstr (n, s) is almost equivalent to
  * mutt_format_string (bigbuf, big, n, n, 0, ' ', s, big), addstr (bigbuf)
