@@ -36,10 +36,11 @@ static int browse_get_namespace (IMAP_DATA *idata, char *nsbuf, int nsblen,
 static int browse_verify_namespace (IMAP_DATA* idata,
   IMAP_NAMESPACE_INFO* nsi, int nns);
 
+/* imap_browse: IMAP hook into the folder browser, fills out browser_state,
+ *   given a current folder to browse */
 int imap_browse (char* path, struct browser_state* state)
 {
-  CONNECTION *conn;
-  IMAP_DATA *idata;
+  IMAP_DATA* idata;
   char buf[LONG_STRING];
   char nsbuf[LONG_STRING];
   char mbox[LONG_STRING];
@@ -66,42 +67,23 @@ int imap_browse (char* path, struct browser_state* state)
   strfcpy (list_cmd, option (OPTIMAPLSUB) ? "LSUB" : "LIST", sizeof (list_cmd));
 
   if (!(idata = imap_conn_find (&(mx.account), 0)))
-    return -1;
-  conn = idata->conn;
+    goto fail;
 
-  if (mx.mbox[0] == '\0')
+  if (!mx.mbox)
   {
     home_namespace = 1;
     mbox[0] = '\0';		/* Do not replace "" with "INBOX" here */
-    mx.mbox = ImapHomeNamespace;
+    mx.mbox = safe_strdup(ImapHomeNamespace);
     nns = 0;
     if (mutt_bit_isset(idata->capabilities,NAMESPACE))
     {
       mutt_message _("Getting namespaces...");
       if (browse_get_namespace (idata, nsbuf, sizeof (nsbuf), 
 			 nsi, sizeof (nsi),  &nns) != 0)
-	return -1;
+	goto fail;
       if (browse_verify_namespace (idata, nsi, nns) != 0)
-	return -1;
+	goto fail;
     }
-    /* What if you have a shared namespace of ""? You'll never be
-     * able to browse it. This isn't conjecture: connect to the Cyrus
-     * reference server (cyrus.andrew.cmu.edu) as anonymous. argh! */
-#if 0
-    if (!mx.mbox)   /* Any explicitly set imap_home_namespace wins */
-    { 
-      for (i = 0; i < nns; i++)
-	if (nsi[i].listable &&
-	    (nsi[i].type == IMAP_NS_PERSONAL || nsi[i].type == IMAP_NS_SHARED))
-	{
-	  mx.mbox = nsi->prefix;
-	  nsi->home_namespace = 1;
-	  break;
-	}
-    }
-    else
-      dprint (4, (debugfile, "Home namespace: %s\n", mx.mbox));
-#endif
   }
 
   mutt_message _("Getting folder list...");
@@ -116,7 +98,7 @@ int imap_browse (char* path, struct browser_state* state)
     strncpy (mbox, buf, sizeof (mbox) - 1);
     n = mutt_strlen (mbox);
 
-    dprint (3, (debugfile, "imap_init_browse: mbox: %s\n", mbox));
+    dprint (3, (debugfile, "imap_browse: mbox: %s\n", mbox));
 
     /* if our target exists and has inferiors, enter it if we
      * aren't already going to */
@@ -126,9 +108,9 @@ int imap_browse (char* path, struct browser_state* state)
       imap_cmd_start (idata, buf);
       do 
       {
-        if (imap_parse_list_response(idata, buf, sizeof(buf), &cur_folder,
-    	    &noselect, &noinferiors, &(idata->delim)) != 0)
-          return -1;
+        if (imap_parse_list_response (idata, &cur_folder, &noselect,
+            &noinferiors, &idata->delim) != 0)
+	  goto fail;
 
         if (cur_folder)
         {
@@ -142,7 +124,7 @@ int imap_browse (char* path, struct browser_state* state)
           }
         }
       }
-      while ((mutt_strncmp (buf, idata->seq, SEQLEN) != 0));
+      while (mutt_strncmp (idata->buf, idata->seq, SEQLEN));
     }
 
     /* if we're descending a folder, mark it as current in browser_state */
@@ -218,14 +200,14 @@ int imap_browse (char* path, struct browser_state* state)
      * server to see if it has descendants. */
     dprint (4, (debugfile, "imap_init_browse: adding INBOX\n"));
     if (browse_add_list_result (idata, "LIST \"\" \"INBOX\"", state, 0))
-      return -1;
+      goto fail;
   }
 
   nsup = state->entrylen;
 
   snprintf (buf, sizeof (buf), "%s \"\" \"%s%%\"", list_cmd, mbox);
   if (browse_add_list_result (idata, buf, state, 0))
-    return -1;
+    goto fail;
 
   qsort(&(state->entry[nsup]),state->entrylen-nsup,sizeof(state->entry[0]),
 	(int (*)(const void*,const void*)) compare_names);
@@ -241,13 +223,65 @@ int imap_browse (char* path, struct browser_state* state)
   }
 
   mutt_clear_error ();
+  FREE (&mx.mbox);
   return 0;
+
+ fail:
+  FREE (&mx.mbox);
+  return -1;
+}
+
+/* imap_mailbox_create: Prompt for a new mailbox name, and try to create it */
+int imap_mailbox_create (const char* folder)
+{
+  IMAP_DATA* idata;
+  IMAP_MBOX mx;
+  char buf[LONG_STRING];
+  short n;
+
+  if (imap_parse_path (folder, &mx) < 0)
+  {
+    dprint (1, (debugfile, "imap_mailbox_create: Bad starting path %s\n",
+      folder));
+    return -1;
+  }
+
+  if (!(idata = imap_conn_find (&mx.account, M_IMAP_CONN_NONEW)))
+  {
+    dprint (1, (debugfile, "imap_mailbox_create: Couldn't find open connection to %s", mx.account.host));
+    goto fail;
+  }
+  
+  strfcpy (buf, NONULL (mx.mbox), sizeof (buf));
+
+  /* append a delimiter if necessary */
+  n = mutt_strlen (buf);
+  if (n && (n < sizeof (buf) - 1) && (buf[n-1] != idata->delim))
+  {
+    buf[n++] = idata->delim;
+    buf[n] = '\0';
+  }
+  
+  if (mutt_get_field (_("Create mailbox: "), buf, sizeof (buf), M_FILE) < 0)
+    goto fail;
+
+  if (imap_create_mailbox (idata, buf) < 0)
+    goto fail;
+
+  mutt_message _("Mailbox created.");
+  sleep (1);
+
+  FREE (&mx.mbox);
+  return 0;
+
+ fail:
+  FREE (&mx.mbox);
+  return -1;
 }
 
 static int browse_add_list_result (IMAP_DATA* idata, const char* cmd,
   struct browser_state* state, short isparent)
 {
-  char buf[LONG_STRING];
   char *name;
   int noselect;
   int noinferiors;
@@ -264,9 +298,12 @@ static int browse_add_list_result (IMAP_DATA* idata, const char* cmd,
 
   do 
   {
-    if (imap_parse_list_response(idata, buf, sizeof(buf), &name,
-        &noselect, &noinferiors, &(idata->delim)) != 0)
+    if (imap_parse_list_response(idata, &name, &noselect, &noinferiors,
+        &idata->delim) != 0)
+    {
+      FREE (&mx.mbox);
       return -1;
+    }
 
     if (name)
     {
@@ -274,13 +311,14 @@ static int browse_add_list_result (IMAP_DATA* idata, const char* cmd,
       if (isparent)
         noselect = 1;
       /* prune current folder from output */
-      if (isparent || strncmp (name, mx.mbox, strlen (name)))
+      if (isparent || mutt_strncmp (name, mx.mbox, strlen (name)))
         imap_add_folder (idata->delim, name, noselect, noinferiors, state,
           isparent);
     }
   }
-  while ((mutt_strncmp (buf, idata->seq, SEQLEN) != 0));
+  while ((mutt_strncmp (idata->buf, idata->seq, SEQLEN) != 0));
 
+  FREE (&mx.mbox);
   return 0;
 }
 
@@ -310,8 +348,8 @@ static void imap_add_folder (char delim, char *folder, int noselect,
   if (isparent)
     strfcpy (relpath, "../", sizeof (relpath));
   /* strip current folder from target, to render a relative path */
-  else if (!strncmp (mx.mbox, folder, strlen (mx.mbox)))
-    strfcpy (relpath, folder + strlen (mx.mbox), sizeof (relpath));
+  else if (!mutt_strncmp (mx.mbox, folder, mutt_strlen (mx.mbox)))
+    strfcpy (relpath, folder + mutt_strlen (mx.mbox), sizeof (relpath));
   else
     strfcpy (relpath, folder, sizeof (relpath));
 
@@ -319,7 +357,10 @@ static void imap_add_folder (char delim, char *folder, int noselect,
    * than at scan, since it's so expensive to scan. But that's big changes
    * to browser.c */
   if (!((regexec (Mask.rx, relpath, 0, NULL, 0) == 0) ^ Mask.not))
+  {
+    FREE (&mx.mbox);
     return;
+  }
 
   imap_qualify_path (tmp, sizeof (tmp), &mx, folder, NULL);
   (state->entry)[state->entrylen].name = safe_strdup (tmp);
@@ -341,6 +382,8 @@ static void imap_add_folder (char delim, char *folder, int noselect,
   (state->entry)[state->entrylen].selectable = !noselect;
   (state->entry)[state->entrylen].inferiors = !noinferiors;
   (state->entrylen)++;
+
+  FREE (&mx.mbox);
 }
 
 static int compare_names(struct folder_file *a, struct folder_file *b) 
@@ -351,13 +394,13 @@ static int compare_names(struct folder_file *a, struct folder_file *b)
 static int browse_get_namespace (IMAP_DATA* idata, char* nsbuf, int nsblen,
   IMAP_NAMESPACE_INFO* nsi, int nsilen, int* nns)
 {
-  char buf[LONG_STRING];
   char *s;
   int n;
   char ns[LONG_STRING];
   char delim = '/';
   int type;
   int nsbused = 0;
+  int rc;
 
   *nns = 0;
   nsbuf[nsblen-1] = '\0';
@@ -366,89 +409,84 @@ static int browse_get_namespace (IMAP_DATA* idata, char* nsbuf, int nsblen,
   
   do 
   {
-    if (mutt_socket_readln (buf, sizeof (buf), idata->conn) < 0)
-      return -1;
+    if ((rc = imap_cmd_step (idata)) != IMAP_CMD_CONTINUE)
+      break;
 
-    if (buf[0] == '*') 
+    s = imap_next_word (idata->buf);
+    if (mutt_strncasecmp ("NAMESPACE", s, 9) == 0)
     {
-      s = imap_next_word (buf);
-      if (mutt_strncasecmp ("NAMESPACE", s, 9) == 0)
+      /* There are three sections to the response, User, Other, Shared,
+       * and maybe more by extension */
+      for (type = IMAP_NS_PERSONAL; *s; type++)
       {
-	/* There are three sections to the response, User, Other, Shared,
-	 * and maybe more by extension */
-	for (type = IMAP_NS_PERSONAL; *s; type++)
+	s = imap_next_word (s);
+	if (*s && strncmp (s, "NIL", 3))
 	{
-	  s = imap_next_word (s);
-	  if (*s && strncmp (s, "NIL", 3))
+	  s++;
+	  while (*s && *s != ')')
 	  {
-	    s++;
-	    while (*s && *s != ')')
-	    {
-	      s++; /* skip ( */
-	      /* copy namespace */
-	      n = 0;
-	      delim = '\0';
+	    s++; /* skip ( */
+	    /* copy namespace */
+	    n = 0;
+	    delim = '\0';
 
-	      if (*s == '\"')
-	      {
-		s++;
-		while (*s && *s != '\"') 
-		{
-		  if (*s == '\\')
-		    s++;
-		  ns[n++] = *s;
-		  s++;
-		}
-	      }
-	      else
-		while (*s && !ISSPACE (*s)) 
-		{
-		  ns[n++] = *s;
-		  s++;
-		}
-	      ns[n] = '\0';
-	      /* delim? */
-	      s = imap_next_word (s);
-	      /* delimiter is meaningless if namespace is "". Why does
-	       * Cyrus provide one?! */
-	      if (n && *s && *s == '\"')
-              {
-		if (s[1] && s[2] == '\"')
-		  delim = s[1];
-		else if (s[1] && s[1] == '\\' && s[2] && s[3] == '\"')
-		  delim = s[2];
-              }
-	      /* skip "" namespaces, they are already listed at the root */
-	      if ((ns[0] != '\0') && (nsbused < nsblen) && (*nns < nsilen))
-	      {
-		dprint (3, (debugfile, "browse_get_namespace: adding %s\n", ns));
-		nsi->type = type;
-		/* Cyrus doesn't append the delimiter to the namespace,
-		 * but UW-IMAP does. We'll strip it here and add it back
-		 * as if it were a normal directory, from the browser */
-		if (n && (ns[n-1] == delim))
-		  ns[--n] = '\0';
-		strncpy(nsbuf+nsbused,ns,nsblen-nsbused-1);
-		nsi->prefix = nsbuf+nsbused;
-		nsbused += n+1;
-		nsi->delim = delim;
-		nsi++;
-		(*nns)++;
-	      }
-	      while (*s && *s != ')') s++;
+	    if (*s == '\"')
+	    {
 	      s++;
+	      while (*s && *s != '\"') 
+	      {
+		if (*s == '\\')
+		  s++;
+		ns[n++] = *s;
+		s++;
+	      }
 	    }
+	    else
+	      while (*s && !ISSPACE (*s)) 
+	      {
+		ns[n++] = *s;
+		s++;
+	      }
+	    ns[n] = '\0';
+	    /* delim? */
+	    s = imap_next_word (s);
+	    /* delimiter is meaningless if namespace is "". Why does
+	     * Cyrus provide one?! */
+	    if (n && *s && *s == '\"')
+	    {
+	      if (s[1] && s[2] == '\"')
+		delim = s[1];
+	      else if (s[1] && s[1] == '\\' && s[2] && s[3] == '\"')
+		delim = s[2];
+	    }
+	    /* skip "" namespaces, they are already listed at the root */
+	    if ((ns[0] != '\0') && (nsbused < nsblen) && (*nns < nsilen))
+	    {
+	      dprint (3, (debugfile, "browse_get_namespace: adding %s\n", ns));
+	      nsi->type = type;
+	      /* Cyrus doesn't append the delimiter to the namespace,
+	       * but UW-IMAP does. We'll strip it here and add it back
+	       * as if it were a normal directory, from the browser */
+	      if (n && (ns[n-1] == delim))
+		ns[--n] = '\0';
+	      strncpy (nsbuf+nsbused,ns,nsblen-nsbused-1);
+	      nsi->prefix = nsbuf+nsbused;
+	      nsbused += n+1;
+	      nsi->delim = delim;
+	      nsi++;
+	      (*nns)++;
+	    }
+	    while (*s && *s != ')') s++;
+	    s++;
 	  }
 	}
       }
-      else
-      {
-	if (imap_handle_untagged (idata, buf) != 0)
-	  return (-1);
-      }
     }
   }
-  while ((mutt_strncmp (buf, idata->seq, SEQLEN) != 0));
+  while (rc == IMAP_CMD_CONTINUE);
+
+  if (rc != IMAP_CMD_DONE)
+    return -1;
 
   return 0;
 }
@@ -481,12 +519,12 @@ static int browse_verify_namespace (IMAP_DATA* idata,
     nsi->home_namespace = 0;
     do 
     {
-      if (imap_parse_list_response(idata, buf, sizeof(buf), &name,
-          &(nsi->noselect), &(nsi->noinferiors), &delim) != 0)
+      if (imap_parse_list_response(idata, &name, &nsi->noselect,
+          &nsi->noinferiors, &delim) != 0)
 	return -1;
       nsi->listable |= (name != NULL);
     }
-    while ((mutt_strncmp (buf, idata->seq, SEQLEN) != 0));
+    while ((mutt_strncmp (idata->buf, idata->seq, SEQLEN) != 0));
   }
 
   return 0;
