@@ -29,25 +29,46 @@
 #include <ctype.h>
 #include <stdlib.h>
 
+#define IMAP_CMD_BUFSIZE 512
+
 /* forward declarations */
+static void cmd_finish (IMAP_DATA* idata);
+static void cmd_handle_fatal (IMAP_DATA* idata);
+static int cmd_handle_untagged (IMAP_DATA* idata);
 static void cmd_make_sequence (char* buf, size_t buflen);
 static void cmd_parse_capabilities (IMAP_DATA* idata, char* s);
 static void cmd_parse_expunge (IMAP_DATA* idata, char* s);
 static void cmd_parse_myrights (IMAP_DATA* idata, char* s);
 
-static char *Capabilities[] = {"IMAP4", "IMAP4rev1", "STATUS", "ACL", 
-  "NAMESPACE", "AUTH=CRAM-MD5", "AUTH=KERBEROS_V4", "AUTH=GSSAPI", 
-  "AUTH=LOGIN", "AUTH-LOGIN", "AUTH=PLAIN", "AUTH=SKEY", "IDLE", 
-  "LOGIN-REFERRALS", "MAILBOX-REFERRALS", "QUOTA", "SCAN", "SORT", 
-  "THREAD=ORDEREDSUBJECT", "UIDPLUS", "AUTH=ANONYMOUS", NULL};
+static char *Capabilities[] = {
+  "IMAP4",
+  "IMAP4rev1",
+  "STATUS",
+  "ACL", 
+  "NAMESPACE",
+  "AUTH=CRAM-MD5",
+  "AUTH=GSSAPI",
+  "AUTH=ANONYMOUS",
+  "STARTTLS",
+  "LOGINDISABLED",
+
+  NULL
+};
 
 /* imap_cmd_start: Given an IMAP command, send it to the server.
  *   Currently a minor convenience, but helps to route all IMAP commands
  *   through a single interface. */
-void imap_cmd_start (IMAP_DATA* idata, const char* cmd)
+int imap_cmd_start (IMAP_DATA* idata, const char* cmd)
 {
   char* out;
   int outlen;
+  int rc = 0;
+
+  if (idata->status == IMAP_FATAL)
+  {
+    cmd_handle_fatal (idata);
+    return IMAP_CMD_FAIL;
+  }
 
   cmd_make_sequence (idata->seq, sizeof (idata->seq));
   /* seq, space, cmd, \r\n\0 */
@@ -55,56 +76,80 @@ void imap_cmd_start (IMAP_DATA* idata, const char* cmd)
   out = (char*) safe_malloc (outlen);
   snprintf (out, outlen, "%s %s\r\n", idata->seq, cmd);
 
-  mutt_socket_write (idata->conn, out);
+  rc = mutt_socket_write (idata->conn, out);
 
-  safe_free ((void**) &out);
+  FREE (&out);
+
+  return (rc < 0) ? IMAP_CMD_FAIL : 0;
 }
 
-/* imap_cmd_finish: When the caller has finished reading command responses,
- *   it must call this routine to perform cleanup (eg fetch new mail if
- *   detected, do expunge) */
-void imap_cmd_finish (IMAP_DATA* idata)
+/* imap_cmd_step: Reads server responses from an IMAP command, detects
+ *   tagged completion response, handles untagged messages, can read
+ *   arbitrarily large strings (using malloc, so don't make it _too_
+ *   large!). */
+int imap_cmd_step (IMAP_DATA* idata)
 {
-  if (!(idata->state == IMAP_SELECTED) || idata->ctx->closing)
+  unsigned int len = 0;
+  int c;
+
+  if (idata->status == IMAP_FATAL)
   {
-    idata->status = 0;
-    mutt_clear_error ();
-    return;
+    cmd_handle_fatal (idata);
+    return IMAP_CMD_FAIL;
+  }
+
+  /* read into buffer, expanding buffer as necessary until we have a full
+   * line */
+  do
+  {
+    if (len == idata->blen)
+    {
+      safe_realloc ((void**) &idata->buf, idata->blen + IMAP_CMD_BUFSIZE);
+      idata->blen = idata->blen + IMAP_CMD_BUFSIZE;
+      dprint (3, (debugfile, "imap_cmd_step: grew buffer to %u bytes\n", idata->blen));
+    }
+
+    if ((c = mutt_socket_readln (idata->buf + len, idata->blen - len,
+      idata->conn)) < 0)
+    {
+      dprint (1, (debugfile, "imap_cmd_step: Error while reading server response, closing connection.\n"));
+      mutt_socket_close (idata->conn);
+      idata->status = IMAP_FATAL;
+      return IMAP_CMD_FAIL;
+    }
+
+    len += c;
+  }
+  /* if we've read all the way to the end of the buffer, we haven't read a
+   * full line (mutt_socket_readln strips the \r, so we always have at least
+   * one character free when we've read a full line) */
+  while (len == idata->blen);
+
+  /* don't let one large string make idata->buf hog memory forever */
+  if ((idata->blen > IMAP_CMD_BUFSIZE) && (len <= IMAP_CMD_BUFSIZE))
+  {
+    safe_realloc ((void**) &idata->buf, IMAP_CMD_BUFSIZE);
+    idata->blen = IMAP_CMD_BUFSIZE;
+    dprint (3, (debugfile, "imap_cmd_step: shrank buffer to %u bytes\n", idata->blen));
   }
   
-  if ((idata->status == IMAP_NEW_MAIL || 
-       (idata->reopen & (IMAP_REOPEN_PENDING|IMAP_NEWMAIL_PENDING)))
-      && (idata->reopen & IMAP_REOPEN_ALLOW))
+  /* handle untagged messages. The caller still gets its shot afterwards. */
+  if (!strncmp (idata->buf, "* ", 2) &&
+      cmd_handle_untagged (idata))
+    return IMAP_CMD_FAIL;
+
+  /* server demands a continuation response from us */
+  if (!strncmp (idata->buf, "+ ", 2))
+    return IMAP_CMD_RESPOND;
+
+  /* tagged completion code */
+  if (!mutt_strncmp (idata->buf, idata->seq, SEQLEN))
   {
-    int count = idata->newMailCount;
-
-    if (!(idata->reopen & IMAP_REOPEN_PENDING) &&
-	((idata->status == IMAP_NEW_MAIL) || (idata->reopen & IMAP_NEWMAIL_PENDING))  
-	&& count > idata->ctx->msgcount)
-    {
-      /* read new mail messages */
-      dprint (1, (debugfile, "imap_cmd_finish: fetching new mail\n"));
-
-      count = imap_read_headers (idata->ctx, idata->ctx->msgcount, count-1)+1;
-      idata->check_status = IMAP_NEW_MAIL;
-      idata->reopen &= ~IMAP_NEWMAIL_PENDING;
-    }
-    else
-    {
-      imap_expunge_mailbox (idata);
-      idata->check_status = IMAP_REOPENED;
-      idata->reopen &= ~(IMAP_REOPEN_PENDING|IMAP_NEWMAIL_PENDING);
-    }
-
-  }
-  else if (!(idata->reopen & IMAP_REOPEN_ALLOW))
-  {
-    if (idata->status == IMAP_NEW_MAIL)
-      idata->reopen |= IMAP_NEWMAIL_PENDING;
+    cmd_finish (idata);
+    return imap_code (idata->buf) ? IMAP_CMD_DONE : IMAP_CMD_NO;
   }
 
-  idata->status = 0;
-  mutt_clear_error ();
+  return IMAP_CMD_CONTINUE;
 }
 
 /* imap_code: returns 1 if the command result was OK, or 0 if NO or BAD */
@@ -123,11 +168,17 @@ int imap_code (const char *s)
  *   IMAP_CMD_PASS: command contains a password. Suppress logging.
  * Return 0 on success, -1 on Failure, -2 on OK Failure
  */
-int imap_exec (char* buf, size_t buflen, IMAP_DATA* idata, const char* cmd,
-  int flags)
+int imap_exec (IMAP_DATA* idata, const char* cmd, int flags)
 {
   char* out;
   int outlen;
+  int rc;
+
+  if (idata->status == IMAP_FATAL)
+  {
+    cmd_handle_fatal (idata);
+    return -1;
+  }
 
   /* create sequence for command */
   cmd_make_sequence (idata->seq, sizeof (idata->seq));
@@ -136,36 +187,32 @@ int imap_exec (char* buf, size_t buflen, IMAP_DATA* idata, const char* cmd,
   out = (char*) safe_malloc (outlen);
   snprintf (out, outlen, "%s %s\r\n", idata->seq, cmd);
 
-  mutt_socket_write_d (idata->conn, out,
+  rc = mutt_socket_write_d (idata->conn, out,
     flags & IMAP_CMD_PASS ? IMAP_LOG_PASS : IMAP_LOG_CMD);
-
   safe_free ((void**) &out);
 
+  if (rc < 0)
+    return -1;
+
   do
-  {
-    if (mutt_socket_readln (buf, buflen, idata->conn) < 0)
-      return -1;
+    rc = imap_cmd_step (idata);
+  while (rc == IMAP_CMD_CONTINUE);
 
-    if (buf[0] == '*' && imap_handle_untagged (idata, buf) != 0)
-      return -1;
-  }
-  while (mutt_strncmp (buf, idata->seq, SEQLEN) != 0);
+  if (rc != IMAP_CMD_DONE)
+    return -1;
 
-  imap_cmd_finish (idata);
-
-  if (!imap_code (buf))
+  if (!imap_code (idata->buf))
   {
     char *pc;
 
     if (flags & IMAP_CMD_FAIL_OK)
       return -2;
 
-    dprint (1, (debugfile, "imap_exec: command failed: %s\n", buf));
-    pc = buf + SEQLEN;
-    SKIPWS (pc);
+    dprint (1, (debugfile, "imap_exec: command failed: %s\n", idata->buf));
+    pc = idata->buf;
     pc = imap_next_word (pc);
     mutt_error ("%s", pc);
-    sleep (1);
+    sleep (2);
 
     return -1;
   }
@@ -173,13 +220,62 @@ int imap_exec (char* buf, size_t buflen, IMAP_DATA* idata, const char* cmd,
   return 0;
 }
 
-/* imap_handle_untagged: fallback parser for otherwise unhandled messages. */
-int imap_handle_untagged (IMAP_DATA *idata, char *s)
+/* cmd_finish: When the caller has finished reading command responses,
+ *   it must call this routine to perform cleanup (eg fetch new mail if
+ *   detected, do expunge). Called automatically by imap_cmd_step */
+static void cmd_finish (IMAP_DATA* idata)
 {
-  char *pn;
+  if (!(idata->state == IMAP_SELECTED) || idata->ctx->closing)
+  {
+    mutt_clear_error ();
+    return;
+  }
+  
+  if ((idata->reopen & IMAP_REOPEN_ALLOW) &&
+      (idata->reopen & (IMAP_EXPUNGE_PENDING|IMAP_NEWMAIL_PENDING)))
+  {
+    int count = idata->newMailCount;
+
+    if (!(idata->reopen & IMAP_EXPUNGE_PENDING) &&
+	(idata->reopen & IMAP_NEWMAIL_PENDING)
+	&& count > idata->ctx->msgcount)
+    {
+      /* read new mail messages */
+      dprint (2, (debugfile, "cmd_finish: Fetching new mail\n"));
+      /* check_status: curs_main uses imap_check_mailbox to detect
+       *   whether the index needs updating */
+      idata->check_status = IMAP_NEWMAIL_PENDING;
+      idata->reopen &= ~IMAP_NEWMAIL_PENDING;
+      count = imap_read_headers (idata, idata->ctx->msgcount, count-1)+1;
+    }
+    else
+    {
+      dprint (2, (debugfile, "cmd_finish: Expunging mailbox\n"));
+      imap_expunge_mailbox (idata);
+      idata->reopen &= ~(IMAP_EXPUNGE_PENDING|IMAP_NEWMAIL_PENDING);
+    }
+  }
+
+  idata->status = 0;
+}
+
+/* cmd_handle_fatal: when IMAP_DATA is in fatal state, do what we can */
+static void cmd_handle_fatal (IMAP_DATA* idata)
+{
+  if ((idata->state == IMAP_SELECTED) &&
+      (idata->reopen & IMAP_REOPEN_ALLOW) &&
+      !idata->ctx->closing)
+    mx_fastclose_mailbox (idata->ctx);
+}
+
+/* cmd_handle_untagged: fallback parser for otherwise unhandled messages. */
+static int cmd_handle_untagged (IMAP_DATA* idata)
+{
+  char* s;
+  char* pn;
   int count;
 
-  s = imap_next_word (s);
+  s = imap_next_word (idata->buf);
 
   if ((idata->state == IMAP_SELECTED) && isdigit (*s))
   {
@@ -191,10 +287,12 @@ int imap_handle_untagged (IMAP_DATA *idata, char *s)
      */
     if (mutt_strncasecmp ("EXISTS", s, 6) == 0)
     {
+      dprint (2, (debugfile, "Handling EXISTS\n"));
+
       /* new mail arrived */
       count = atoi (pn);
 
-      if ( !(idata->reopen & IMAP_REOPEN_PENDING) &&
+      if ( !(idata->reopen & IMAP_EXPUNGE_PENDING) &&
 	   count < idata->ctx->msgcount)
       {
 	/* something is wrong because the server reported fewer messages
@@ -202,22 +300,21 @@ int imap_handle_untagged (IMAP_DATA *idata, char *s)
 	 */
 	mutt_error _("Fatal error.  Message count is out of sync!");
 	idata->status = IMAP_FATAL;
-	mx_fastclose_mailbox (idata->ctx);
 	return -1;
       }
       /* at least the InterChange server sends EXISTS messages freely,
        * even when there is no new mail */
       else if (count == idata->ctx->msgcount)
 	dprint (3, (debugfile,
-          "imap_handle_untagged: superfluous EXISTS message.\n"));
+          "cmd_handle_untagged: superfluous EXISTS message.\n"));
       else
       {
-	if (!(idata->reopen & IMAP_REOPEN_PENDING))
+	if (!(idata->reopen & IMAP_EXPUNGE_PENDING))
         {
           dprint (2, (debugfile,
-            "imap_handle_untagged: New mail in %s - %d messages total.\n",
+            "cmd_handle_untagged: New mail in %s - %d messages total.\n",
             idata->mailbox, count));
-	  idata->status = IMAP_NEW_MAIL;
+	  idata->reopen |= IMAP_NEWMAIL_PENDING;
         }
 	idata->newMailCount = count;
       }
@@ -232,6 +329,12 @@ int imap_handle_untagged (IMAP_DATA *idata, char *s)
     cmd_parse_myrights (idata, s);
   else if (mutt_strncasecmp ("BYE", s, 3) == 0)
   {
+    dprint (2, (debugfile, "Handling BYE\n"));
+
+    /* check if we're logging out */
+    if (idata->status == IMAP_BYE)
+      return 0;
+
     /* server shut down our connection */
     s += 3;
     SKIPWS (s);
@@ -246,13 +349,12 @@ int imap_handle_untagged (IMAP_DATA *idata, char *s)
   }
   else if (option (OPTIMAPSERVERNOISE) && (mutt_strncasecmp ("NO", s, 2) == 0))
   {
+    dprint (2, (debugfile, "Handling untagged NO\n"));
+
     /* Display the warning message from the server */
     mutt_error ("%s", s+3);
     sleep (2);
   }
-  else
-    dprint (1, (debugfile, "imap_handle_untagged(): unhandled request: %s\n",
-      s));
 
   return 0;
 }
@@ -274,6 +376,8 @@ static void cmd_parse_capabilities (IMAP_DATA* idata, char* s)
 {
   int x;
 
+  dprint (2, (debugfile, "Handling CAPABILITY\n"));
+
   idata->capstr = safe_strdup (imap_next_word (s));
   
   while (*s) 
@@ -295,6 +399,8 @@ static void cmd_parse_expunge (IMAP_DATA* idata, char* s)
   int expno, cur;
   HEADER* h;
 
+  dprint (2, (debugfile, "Handling EXPUNGE\n"));
+
   expno = atoi (s);
 
   /* walk headers, zero seqno of expunged message, decrement seqno of those
@@ -305,18 +411,20 @@ static void cmd_parse_expunge (IMAP_DATA* idata, char* s)
   {
     h = idata->ctx->hdrs[cur];
 
-    if (HEADER_DATA (h)->sid == expno)
-      HEADER_DATA (h)->sid = 0;
-    else if (HEADER_DATA (h)->sid > expno)
-      HEADER_DATA (h)->sid--;
+    if (h->index+1 == expno)
+      h->index = -1;
+    else if (h->index+1 > expno)
+      h->index--;
   }
 
-  idata->reopen |= IMAP_REOPEN_PENDING;
+  idata->reopen |= IMAP_EXPUNGE_PENDING;
 }
 
 /* cmd_parse_myrights: set rights bits according to MYRIGHTS response */
 static void cmd_parse_myrights (IMAP_DATA* idata, char* s)
 {
+  dprint (2, (debugfile, "Handling MYRIGHTS\n"));
+
   s = imap_next_word (s);
   s = imap_next_word (s);
 
