@@ -45,18 +45,16 @@ static int imap_check_capabilities (IMAP_DATA* idata);
 static void imap_set_flag (IMAP_DATA* idata, int aclbit, int flag,
   const char* str, char* flags);
 
-int imap_create_mailbox (CONTEXT* ctx, char* mailbox)
+int imap_create_mailbox (IMAP_DATA* idata, char* mailbox)
 {
   char buf[LONG_STRING], mbox[LONG_STRING];
 
   imap_munge_mbox_name (mbox, sizeof (mbox), mailbox);
   snprintf (buf, sizeof (buf), "CREATE %s", mbox);
       
-  if (imap_exec (buf, sizeof (buf), CTX_DATA, buf, 0) != 0)
-  {
-    imap_error ("imap_create_mailbox()", buf);
+  if (imap_exec (idata, buf, 0) != 0)
     return -1;
-  }
+
   return 0;
 }
 
@@ -64,10 +62,10 @@ int imap_delete_mailbox (CONTEXT* ctx, char* mailbox)
 {
   char buf[LONG_STRING], mbox[LONG_STRING];
   
-  imap_quote_string (mbox, sizeof (mbox), mailbox);
+  imap_munge_mbox_name (mbox, sizeof (mbox), mailbox);
   snprintf (buf, sizeof (buf), "DELETE %s", mbox);
 
-  if (imap_exec (buf, sizeof (buf), CTX_DATA, buf, 0) != 0)
+  if (imap_exec ((IMAP_DATA*) ctx->data, buf, 0) != 0)
     return -1;
 
   return 0;
@@ -192,8 +190,7 @@ int imap_read_literal (FILE* fp, IMAP_DATA* idata, long bytes)
 
 /* imap_expunge_mailbox: Purge IMAP portion of expunged messages from the
  *   context. Must not be done while something has a handle on any headers
- *   (eg inside pager or editor). mx_update_tables and mutt_sort_headers
- *   must be called afterwards. */
+ *   (eg inside pager or editor). That is, check IMAP_REOPEN_ALLOW. */
 void imap_expunge_mailbox (IMAP_DATA* idata)
 {
   HEADER* h;
@@ -203,7 +200,7 @@ void imap_expunge_mailbox (IMAP_DATA* idata)
   {
     h = idata->ctx->hdrs[i];
 
-    if (!HEADER_DATA(h)->sid)
+    if (h->index == -1)
     {
       dprint (2, (debugfile, "Expunging message UID %d.\n", HEADER_DATA (h)->uid));
 
@@ -221,216 +218,17 @@ void imap_expunge_mailbox (IMAP_DATA* idata)
       imap_free_header_data (&h->data);
     }
   }
+
+  /* We may be called on to expunge at any time. We can't rely on the caller
+   * to always know to rethread */
+  mx_update_tables (idata->ctx, 0);
+  mutt_sort_headers (idata->ctx, 1);
 }
 
-#if 0
-/* imap_reopen_mailbox: Reopen an imap mailbox.  This is used when the
- * server sends an EXPUNGE message, indicating that some messages may have
- * been deleted. This is a heavy handed approach, as it reparses all of the
- * headers, but it should guarantee correctness.  Later, we might implement
- * something to actually only remove the messages that are marked
- * EXPUNGE.
- */
-int imap_reopen_mailbox (IMAP_DATA* idata)
-{
-  HEADER **old_hdrs;
-  CONTEXT* ctx;
-  int old_msgcount;
-  char buf[LONG_STRING];
-  char bufout[LONG_STRING];
-  char *pc = NULL;
-  int count = 0;
-  int msg_mod = 0;
-  int n;
-  int i, j;
-  int index_hint_set;
-
-  ctx = idata->ctx;
-
-  ctx->quiet = 1;
-
-  if (Sort != SORT_ORDER)
-  {
-    short old_sort;
-
-    old_sort = Sort;
-    Sort = SORT_ORDER;
-    mutt_sort_headers (ctx, 1);
-    Sort = old_sort;
-  }
-
-  old_hdrs = NULL;
-  old_msgcount = 0;
-
-  /* simulate a close */
-  hash_destroy (&ctx->id_hash, NULL);
-  hash_destroy (&ctx->subj_hash, NULL);
-  safe_free ((void **) &ctx->v2r);
-  if (ctx->readonly)
-  {
-    for (i = 0; i < ctx->msgcount; i++)
-      mutt_free_header (&(ctx->hdrs[i])); /* nothing to do! */
-    safe_free ((void **) &ctx->hdrs);
-  }
-  else
-  {
-    /* save the old headers */
-    old_msgcount = ctx->msgcount;
-    old_hdrs = ctx->hdrs;
-    ctx->hdrs = NULL;
-  }
-
-  ctx->hdrmax = 0;      /* force allocation of new headers */
-  ctx->msgcount = 0;
-  ctx->vcount = 0;
-  ctx->tagged = 0;
-  ctx->deleted = 0;
-  ctx->new = 0;
-  ctx->unread = 0;
-  ctx->flagged = 0;
-  ctx->changed = 0;
-  ctx->id_hash = hash_create (1031);
-  ctx->subj_hash = hash_create (1031);
-
-  mutt_message (_("Reopening mailbox... %s"), CTX_DATA->mailbox);
-  imap_munge_mbox_name (buf, sizeof (buf), CTX_DATA->mailbox);
-  snprintf (bufout, sizeof (bufout), "STATUS %s (MESSAGES)", buf);
-
-  imap_cmd_start (CTX_DATA, bufout);
-
-  do
-  {
-    if (mutt_socket_readln (buf, sizeof (buf), CTX_DATA->conn) < 0)
-      break;
-
-    if (buf[0] == '*')
-    {
-      pc = buf + 2;
-
-      if (!mutt_strncasecmp ("STATUS", pc, 6) &&
-	  (pc = (char *) mutt_stristr (pc, "MESSAGES")))
-      {
-	char* pn;
-	
-	/* skip "MESSAGES" */
-	pc += 8;
-	SKIPWS (pc);
-	pn = pc;
-
-	while (*pc && isdigit (*pc))
-	  pc++;
-	*pc++ = 0;
-	n = atoi (pn);
-	count = n;
-      }
-      else if (imap_handle_untagged (CTX_DATA, buf) != 0)
-	return -1;
-    }
-  }
-  while (mutt_strncmp (CTX_DATA->seq, buf, mutt_strlen (CTX_DATA->seq)) != 0);
-
-  if (!imap_code (buf))
-  {
-    char *s;
-    s = imap_next_word (buf); /* skip seq */
-    s = imap_next_word (s); /* Skip response */
-    mutt_error ("%s", s);
-    sleep (1);
-    return -1;
-  }
-
-  ctx->hdrmax = count;
-  ctx->hdrs = safe_malloc (count * sizeof (HEADER *));
-  ctx->v2r = safe_malloc (count * sizeof (int));
-  ctx->msgcount = 0;
-  count = imap_read_headers (ctx, 0, count - 1) + 1;
-
-  index_hint_set = 1;
-
-  if (!ctx->readonly)
-  {
-    for (i = 0; i < ctx->msgcount; i++)
-    {
-      int found = 0;
-
-      /* some messages have been deleted, and new  messages have been
-       * appended at the end; the heuristic is that old messages have then
-       * "advanced" towards the beginning of the folder, so we begin the
-       * search at index "i"
-       */
-      for (j = i; j < old_msgcount; j++)
-      {
-	if (old_hdrs[j] == NULL)
-	  continue;
-	if (mbox_strict_cmp_headers (ctx->hdrs[i], old_hdrs[j]))
-	{
-	  found = 1;
-	  break;
-	}
-      }
-      if (!found)
-      {
-	for (j = 0; j < i && j < old_msgcount; j++)
-	{
-	  if (old_hdrs[j] == NULL)
-	    continue;
-	  if (mbox_strict_cmp_headers (ctx->hdrs[i], old_hdrs[j]))
-	  {
-	    found = 1;
-	    break;
-	  }
-	}
-      }
-      if (found)
-      {
-	/* this is best done here */
-/*
-	if (!index_hint_set && *index_hint == j)
-	  *index_hint = i;
-*/
-
-	if (old_hdrs[j]->changed)
-	{
-	  /* Only update the flags if the old header was changed;
-	   * otherwise, the header may have been modified
-	   * externally, and we don't want to lose _those_ changes 
-	   */
-	  
-	  mutt_set_flag (ctx, ctx->hdrs[i], M_FLAG, old_hdrs[j]->flagged);
-	  mutt_set_flag (ctx, ctx->hdrs[i], M_REPLIED, old_hdrs[j]->replied);
-	  mutt_set_flag (ctx, ctx->hdrs[i], M_OLD, old_hdrs[j]->old);
-	  mutt_set_flag (ctx, ctx->hdrs[i], M_READ, old_hdrs[j]->read);
-	}
-	mutt_set_flag (ctx, ctx->hdrs[i], M_DELETE, old_hdrs[j]->deleted);
-	mutt_set_flag (ctx, ctx->hdrs[i], M_TAG, old_hdrs[j]->tagged);
-
-	/* we don't need this header any more */
-	mutt_free_header (&(old_hdrs[j]));
-      }
-    }
-
-    /* free the remaining old headers */
-    for (j = 0; j < old_msgcount; j++)
-    {
-      if (old_hdrs[j])
-      {
-	mutt_free_header (&(old_hdrs[j]));
-	msg_mod = 1;
-      }
-    }
-    safe_free ((void **) &old_hdrs);
-  }
-
-  ctx->quiet = 0;
-
-  return 0;
-}
-#endif
-
 static int imap_get_delim (IMAP_DATA *idata)
 {
-  char buf[LONG_STRING];
   char *s;
+  int rc;
 
   /* assume that the delim is /.  If this fails, we're in bigger trouble
    * than getting the delim wrong */
@@ -440,31 +238,31 @@ static int imap_get_delim (IMAP_DATA *idata)
 
   do 
   {
-    if (mutt_socket_readln (buf, sizeof (buf), idata->conn) < 0)
-      return -1;
+    if ((rc = imap_cmd_step (idata)) != IMAP_CMD_CONTINUE)
+      break;
 
-    if (buf[0] == '*') 
+    s = imap_next_word (idata->buf);
+    if (mutt_strncasecmp ("LIST", s, 4) == 0)
     {
-      s = imap_next_word (buf);
-      if (mutt_strncasecmp ("LIST", s, 4) == 0)
-      {
-	s = imap_next_word (s);
-	s = imap_next_word (s);
-	if (s && s[0] == '\"' && s[1] && s[2] == '\"')
-	  idata->delim = s[1];
-	else if (s && s[0] == '\"' && s[1] && s[1] == '\\' && s[2] && s[3] == '\"')
-	  idata->delim = s[2];
-      }
-      else
-      {
-	if (imap_handle_untagged (idata, buf) != 0)
-	  return -1;
-      }
+      s = imap_next_word (s);
+      s = imap_next_word (s);
+      if (s && s[0] == '\"' && s[1] && s[2] == '\"')
+	idata->delim = s[1];
+      else if (s && s[0] == '\"' && s[1] && s[1] == '\\' && s[2] && s[3] == '\"')
+	idata->delim = s[2];
     }
   }
-  while ((mutt_strncmp (buf, idata->seq, SEQLEN) != 0));
+  while (rc == IMAP_CMD_CONTINUE);
 
-  return 0;
+  if (rc != IMAP_CMD_DONE)
+  {
+    dprint (1, (debugfile, "imap_get_delim: failed.\n"));
+    return -1;
+  }
+
+  dprint (2, (debugfile, "Delimiter: %c\n", idata->delim));
+
+  return -1;
 }
 
 /* get rights for folder, let imap_handle_untagged do the rest */
@@ -475,7 +273,7 @@ static int imap_check_acl (IMAP_DATA *idata)
 
   imap_munge_mbox_name (mbox, sizeof(mbox), idata->mailbox);
   snprintf (buf, sizeof (buf), "MYRIGHTS %s", mbox);
-  if (imap_exec (buf, sizeof (buf), idata, buf, 0) != 0)
+  if (imap_exec (idata, buf, 0) != 0)
   {
     imap_error ("imap_check_acl", buf);
     return -1;
@@ -486,11 +284,9 @@ static int imap_check_acl (IMAP_DATA *idata)
 /* imap_check_capabilities: make sure we can log in to this server. */
 static int imap_check_capabilities (IMAP_DATA* idata)
 {
-  char buf[LONG_STRING];
-
-  if (imap_exec (buf, sizeof (buf), idata, "CAPABILITY", 0) != 0)
+  if (imap_exec (idata, "CAPABILITY", 0) != 0)
   {
-    imap_error ("imap_check_capabilities", buf);
+    imap_error ("imap_check_capabilities", idata->buf);
     return -1;
   }
 
@@ -514,7 +310,9 @@ IMAP_DATA* imap_conn_find (const ACCOUNT* account, int flags)
   IMAP_DATA* idata;
   ACCOUNT* creds;
 
-  conn = mutt_conn_find (NULL, account);
+  if (!(conn = mutt_conn_find (NULL, account)))
+    return NULL;
+
   /* if opening a new UNSELECTED connection, preserve existing creds */
   creds = &(conn->account);
 
@@ -522,11 +320,11 @@ IMAP_DATA* imap_conn_find (const ACCOUNT* account, int flags)
   if (flags & M_IMAP_CONN_NOSELECT)
     while (conn->data && ((IMAP_DATA*) conn->data)->state == IMAP_SELECTED)
     {
-      conn = mutt_conn_find (conn, account);
+      if (!(conn = mutt_conn_find (conn, account)))
+	return NULL;
       memcpy (&(conn->account), creds, sizeof (ACCOUNT));
     }
   
-  
   idata = (IMAP_DATA*) conn->data;
 
   /* don't open a new connection if one isn't wanted */
@@ -566,17 +364,17 @@ int imap_open_connection (IMAP_DATA* idata)
 
   idata->state = IMAP_CONNECTED;
 
-  if (mutt_socket_readln (buf, sizeof (buf), idata->conn) < 0)
+  if (imap_cmd_step (idata) != IMAP_CMD_CONTINUE)
     goto bail;
 
-  if (mutt_strncmp ("* OK", buf, 4) == 0)
+  if (mutt_strncmp ("* OK", idata->buf, 4) == 0)
   {
     if (imap_check_capabilities (idata) || imap_authenticate (idata))
       goto bail;
   }
-  else if (mutt_strncmp ("* PREAUTH", buf, 9) == 0)
+  else if (mutt_strncmp ("* PREAUTH", idata->buf, 9) == 0)
   {
-    if (imap_check_capabilities(idata) != 0)
+    if (imap_check_capabilities (idata) != 0)
       goto bail;
   } 
   else
@@ -662,8 +460,8 @@ int imap_open_mailbox (CONTEXT* ctx)
   char buf[LONG_STRING];
   char bufout[LONG_STRING];
   int count = 0;
-  int n;
   IMAP_MBOX mx;
+  int rc;
   
   if (imap_parse_path (ctx->path, &mx))
   {
@@ -673,11 +471,14 @@ int imap_open_mailbox (CONTEXT* ctx)
 
   /* we require a connection which isn't currently in IMAP_SELECTED state */
   if (!(idata = imap_conn_find (&(mx.account), M_IMAP_CONN_NOSELECT)))
-    return -1;
+    goto fail;
   conn = idata->conn;
 
   /* once again the context is new */
-  ctx->data = (void *) idata;
+  ctx->data = idata;
+
+  if (idata->status == IMAP_FATAL)
+    goto fail;
 
   /* Clean up path and replace the one in the ctx */
   imap_fix_path (idata, mx.mbox, buf, sizeof (buf));
@@ -707,56 +508,53 @@ int imap_open_mailbox (CONTEXT* ctx)
   {
     char *pc;
     
-    if (mutt_socket_readln (buf, sizeof (buf), conn) < 0)
+    if ((rc = imap_cmd_step (idata)) != IMAP_CMD_CONTINUE)
       break;
 
-    if (buf[0] == '*')
+    pc = idata->buf + 2;
+    pc = imap_next_word (pc);
+    if (!mutt_strncasecmp ("EXISTS", pc, 6))
     {
-      pc = buf + 2;
+      /* imap_handle_untagged will have picked up the EXISTS message and
+       * set the NEW_MAIL flag. We clear it here. */
+      idata->reopen &= ~IMAP_NEWMAIL_PENDING;
+      count = idata->newMailCount;
+      idata->newMailCount = 0;
+    }
 
-      if (isdigit (*pc))
-      {
-	char *pn = pc;
+    pc = idata->buf + 2;
 
-	while (*pc && isdigit (*pc))
-	  pc++;
-	*pc++ = '\0';
-	n = atoi (pn);
-	SKIPWS (pc);
-	if (mutt_strncasecmp ("EXISTS", pc, 6) == 0)
-	  count = n;
-      }
-      /* Obtain list of available flags here, may be overridden by a
-       * PERMANENTFLAGS tag in the OK response */
-      else if (mutt_strncasecmp ("FLAGS", pc, 5) == 0)
+    /* Obtain list of available flags here, may be overridden by a
+     * PERMANENTFLAGS tag in the OK response */
+    if (mutt_strncasecmp ("FLAGS", pc, 5) == 0)
+    {
+      /* don't override PERMANENTFLAGS */
+      if (!idata->flags)
       {
-        /* don't override PERMANENTFLAGS */
-        if (!idata->flags)
-        {
-          dprint (2, (debugfile, "Getting mailbox FLAGS\n"));
-          if ((pc = imap_get_flags (&(idata->flags), pc)) == NULL)
-            return -1;
-        }
+	dprint (2, (debugfile, "Getting mailbox FLAGS\n"));
+	if ((pc = imap_get_flags (&(idata->flags), pc)) == NULL)
+	  goto fail;
       }
-      /* PERMANENTFLAGS are massaged to look like FLAGS, then override FLAGS */
-      else if (mutt_strncasecmp ("OK [PERMANENTFLAGS", pc, 18) == 0)
-      {
-        dprint (2, (debugfile,
-          "Getting mailbox PERMANENTFLAGS\n"));
-        /* safe to call on NULL */
-        mutt_free_list (&(idata->flags));
-	/* skip "OK [PERMANENT" so syntax is the same as FLAGS */
-        pc += 13;
-        if ((pc = imap_get_flags (&(idata->flags), pc)) == NULL)
-          return -1;
-      }
-      else if (imap_handle_untagged (idata, buf) != 0)
-	return (-1);
+    }
+    /* PERMANENTFLAGS are massaged to look like FLAGS, then override FLAGS */
+    else if (mutt_strncasecmp ("OK [PERMANENTFLAGS", pc, 18) == 0)
+    {
+      dprint (2, (debugfile, "Getting mailbox PERMANENTFLAGS\n"));
+      /* safe to call on NULL */
+      mutt_free_list (&(idata->flags));
+      /* skip "OK [PERMANENT" so syntax is the same as FLAGS */
+      pc += 13;
+      if ((pc = imap_get_flags (&(idata->flags), pc)) == NULL)
+	goto fail;
     }
   }
-  while (mutt_strncmp (idata->seq, buf, mutt_strlen (idata->seq)) != 0);
+  while (rc == IMAP_CMD_CONTINUE);
+
+  if (rc != IMAP_CMD_DONE)
+    goto fail;
+
   /* check for READ-ONLY notification */
-  if (!strncmp (imap_get_qualifier (buf), "[READ-ONLY]", 11))
+  if (!strncmp (imap_get_qualifier (idata->buf), "[READ-ONLY]", 11))
   {
     dprint (2, (debugfile, "Mailbox is read-only.\n"));
     ctx->readonly = 1;
@@ -785,21 +583,21 @@ int imap_open_mailbox (CONTEXT* ctx)
   }
 #endif
 
-  if (!imap_code (buf))
+  if (!imap_code (idata->buf))
   {
     char *s;
-    s = imap_next_word (buf); /* skip seq */
+    s = imap_next_word (idata->buf); /* skip seq */
     s = imap_next_word (s); /* Skip response */
     mutt_error ("%s", s);
     idata->state = IMAP_AUTHENTICATED;
     sleep (1);
-    return -1;
+    goto fail;
   }
 
   if (mutt_bit_isset (idata->capabilities, ACL))
   {
     if (imap_check_acl (idata))
-      return -1;
+      goto fail;
   }
   /* assume we have all rights if ACL is unavailable */
   else
@@ -818,10 +616,15 @@ int imap_open_mailbox (CONTEXT* ctx)
   ctx->hdrs = safe_malloc (count * sizeof (HEADER *));
   ctx->v2r = safe_malloc (count * sizeof (int));
   ctx->msgcount = 0;
-  count = imap_read_headers (ctx, 0, count - 1) + 1;
+  count = imap_read_headers (idata, 0, count - 1) + 1;
 
   dprint (1, (debugfile, "imap_open_mailbox(): msgcount is %d\n", ctx->msgcount));
+  FREE (&mx.mbox);
   return 0;
+
+ fail:
+  FREE (&mx.mbox);
+  return -1;
 }
 
 int imap_open_mailbox_append (CONTEXT *ctx)
@@ -840,7 +643,7 @@ int imap_open_mailbox_append (CONTEXT *ctx)
    * ctx is brand new and mostly empty */
 
   if (!(idata = imap_conn_find (&(mx.account), 0)))
-    return -1;
+    goto fail;
   conn = idata->conn;
 
   ctx->magic = M_IMAP;
@@ -863,39 +666,41 @@ int imap_open_mailbox_append (CONTEXT *ctx)
     /* STATUS not supported */
     mutt_message _("Unable to append to IMAP mailboxes at this server");
 
-    return -1;
+    goto fail;
   }
 
-  r = imap_exec (buf, sizeof (buf), idata, buf, IMAP_CMD_FAIL_OK);
+  r = imap_exec (idata, buf, IMAP_CMD_FAIL_OK);
   if (r == -2)
   {
     /* command failed cause folder doesn't exist */
     snprintf (buf, sizeof (buf), _("Create %s?"), mailbox);
     if (option (OPTCONFIRMCREATE) && mutt_yesorno (buf, 1) < 1)
-      return -1;
+      goto fail;
 
-    if (imap_create_mailbox (ctx, mailbox) < 0)
-      return -1;
+    if (imap_create_mailbox (idata, mailbox) < 0)
+      goto fail;
   }
   else if (r == -1)
     /* Hmm, some other failure */
-    return -1;
+    goto fail;
 
+  FREE (&mx.mbox);
   return 0;
+
+ fail:
+  FREE (&mx.mbox);
+  return -1;
 }
 
+/* imap_logout: Gracefully log out of server. */
 void imap_logout (IMAP_DATA* idata)
 {
-  char buf[LONG_STRING];
-
+  /* we set status here to let imap_handle_untagged know we _expect_ to
+   * receive a bye response (so it doesn't freak out and close the conn) */
+  idata->status = IMAP_BYE;
   imap_cmd_start (idata, "LOGOUT");
-
-  do
-  {
-    if (mutt_socket_readln (buf, sizeof (buf), idata->conn) < 0)
-      break;
-  }
-  while (mutt_strncmp (idata->seq, buf, SEQLEN) != 0);
+  while (imap_cmd_step (idata) == IMAP_CMD_CONTINUE)
+    ;
 }
 
 int imap_close_connection (CONTEXT *ctx)
@@ -1070,7 +875,7 @@ int imap_sync_mailbox (CONTEXT* ctx, int expunge, int* index_hint)
       mutt_message (_("Marking %d messages deleted..."), deleted);
       snprintf (tmp, sizeof (tmp), "UID STORE %s +FLAGS.SILENT (\\Deleted)",
         buf);
-      if (imap_exec (buf, sizeof (buf), idata, tmp, 0) != 0)
+      if (imap_exec (idata, tmp, 0) != 0)
         /* continue, let regular store try before giving up */
         dprint(2, (debugfile, "imap_sync_mailbox: fast delete failed\n"));
       else
@@ -1116,19 +921,20 @@ int imap_sync_mailbox (CONTEXT* ctx, int expunge, int* index_hint)
 
         mutt_remove_trailing_ws (flags);
 
-        snprintf (buf, sizeof (buf), "STORE %d -FLAGS.SILENT (%s)",
-          ctx->hdrs[n]->index + 1, flags);
+        snprintf (buf, sizeof (buf), "UID STORE %d -FLAGS.SILENT (%s)",
+          HEADER_DATA (ctx->hdrs[n])->uid, flags);
       }
       else
-        snprintf (buf, sizeof (buf), "STORE %d FLAGS.SILENT (%s)",
-          ctx->hdrs[n]->index + 1, flags);
+        snprintf (buf, sizeof (buf), "UID STORE %d FLAGS.SILENT (%s)",
+          HEADER_DATA (ctx->hdrs[n])->uid, flags);
 
       /* after all this it's still possible to have no flags, if you
        * have no ACL rights */
-      if (*flags && (imap_exec (buf, sizeof (buf), idata, buf, 0) != 0) &&
+      if (*flags && (imap_exec (idata, buf, 0) != 0) &&
         (err_continue != M_YES))
       {
-        err_continue = imap_continue ("imap_sync_mailbox: STORE failed", buf);
+        err_continue = imap_continue ("imap_sync_mailbox: STORE failed",
+          idata->buf);
         if (err_continue != M_YES)
           return -1;
       }
@@ -1143,9 +949,9 @@ int imap_sync_mailbox (CONTEXT* ctx, int expunge, int* index_hint)
       mutt_bit_isset(idata->rights, IMAP_ACL_DELETE))
   {
     mutt_message _("Expunging messages from server...");
-    if (imap_exec (buf, sizeof (buf), CTX_DATA, "EXPUNGE", 0) != 0)
+    if (imap_exec (idata, "EXPUNGE", 0) != 0)
     {
-      imap_error ("imap_sync_mailbox: EXPUNGE failed", buf);
+      imap_error ("imap_sync_mailbox: EXPUNGE failed", idata->buf);
       return -1;
     }
   }
@@ -1157,7 +963,6 @@ int imap_sync_mailbox (CONTEXT* ctx, int expunge, int* index_hint)
 void imap_close_mailbox (CONTEXT* ctx)
 {
   IMAP_DATA* idata;
-  char buf[LONG_STRING];
   int i;
 
   idata = (IMAP_DATA*) ctx->data;
@@ -1165,17 +970,14 @@ void imap_close_mailbox (CONTEXT* ctx)
   if (!idata)
     return;
 
-  idata->reopen &= IMAP_REOPEN_ALLOW;
-
-  if ((idata->state == IMAP_SELECTED) && (ctx == idata->ctx))
+  if ((idata->status != IMAP_FATAL) &&
+      (idata->state == IMAP_SELECTED) &&
+      (ctx == idata->ctx))
   {
-    if (!(idata->noclose))
-    {
-      mutt_message _("Closing mailbox...");
-      if (imap_exec (buf, sizeof (buf), idata, "CLOSE", 0) != 0)
-        imap_error ("CLOSE failed", buf);
-    }
-    
+    if (!(idata->noclose) && imap_exec (idata, "CLOSE", 0))
+      imap_error ("CLOSE failed", idata->buf);
+
+    idata->reopen &= IMAP_REOPEN_ALLOW;
     idata->state = IMAP_AUTHENTICATED;
     FREE (&(idata->mailbox));
   }
@@ -1197,45 +999,40 @@ void imap_close_mailbox (CONTEXT* ctx)
 /* use the NOOP command to poll for new mail
  *
  * return values:
- *	M_REOPENED	mailbox has been reopened
+ *	M_REOPENED	mailbox has been externally modified
  *	M_NEW_MAIL	new mail has arrived!
  *	0		no change
  *	-1		error
  */
 int imap_check_mailbox (CONTEXT *ctx, int *index_hint)
 {
-  char buf[LONG_STRING];
-  static time_t checktime=0;
-  time_t t = 0;
+  /* overload keyboard timeout to avoid many mailbox checks in a row.
+   * Most users don't like having to wait exactly when they press a key. */
+  static time_t LastCheck = 0;
 
-  /* 
-   * gcc thinks it has to warn about uninitialized use
-   * of t.  This is wrong.
-   */
-  
-  if (ImapCheckTimeout)
-  { 
-    t = time(NULL);
-    t -= checktime;
-  }
+  IMAP_DATA* idata;
+  time_t now;
 
-  if ((ImapCheckTimeout && t >= ImapCheckTimeout)
-      || ((CTX_DATA->reopen & IMAP_REOPEN_ALLOW) && (CTX_DATA->reopen & ~IMAP_REOPEN_ALLOW)))
-  {
-    if (ImapCheckTimeout) checktime += t;
+  idata = (IMAP_DATA*) ctx->data;
 
-    CTX_DATA->check_status = 0;
-    if (imap_exec (buf, sizeof (buf), CTX_DATA, "NOOP", 0) != 0)
+  now = time(NULL);
+  if (now > LastCheck + Timeout) {
+    LastCheck = now;
+
+    if (imap_exec (idata, "NOOP", 0) != 0)
     {
-      imap_error ("imap_check_mailbox()", buf);
+      imap_error ("imap_check_mailbox", idata->buf);
       return -1;
     }
-    
-    if (CTX_DATA->check_status == IMAP_NEW_MAIL)
-      return M_NEW_MAIL;
-    if (CTX_DATA->check_status == IMAP_REOPENED)
-      return M_REOPENED;
   }
+    
+  if (idata->check_status & IMAP_NEWMAIL_PENDING)
+  {
+    idata->check_status &= ~IMAP_NEWMAIL_PENDING;
+    return M_NEW_MAIL;
+  }
+  /* TODO: we should be able to detect external changes and return
+   *   M_REOPENED here. */
   
   return 0;
 }
@@ -1257,6 +1054,7 @@ int imap_mailbox_check (char* path, int new)
   int msgcount = 0;
   int connflags = 0;
   IMAP_MBOX mx;
+  int rc;
   
   if (imap_parse_path (path, &mx))
     return -1;
@@ -1266,13 +1064,14 @@ int imap_mailbox_check (char* path, int new)
     connflags = M_IMAP_CONN_NONEW;
 
   if (!(idata = imap_conn_find (&(mx.account), connflags)))
+  {
+    FREE (&mx.mbox);
     return -1;
+  }
   conn = idata->conn;
 
   imap_fix_path (idata, mx.mbox, buf, sizeof (buf));
-  /* Update the path, if it fits */
-  if (strlen (buf) < strlen (mx.mbox))
-      strcpy (mx.mbox, buf);
+  FREE (&mx.mbox);
 
   imap_munge_mbox_name (mbox, sizeof(mbox), buf);
   strfcpy (mbox_unquoted, buf, sizeof (mbox_unquoted));
@@ -1294,129 +1093,115 @@ int imap_mailbox_check (char* path, int new)
       new ? "RECENT" : "MESSAGES");
   }
   else
-  {
     /* Server does not support STATUS, and this is not the current mailbox.
      * There is no lightweight way to check recent arrivals */
-      return -1;
-  }
+    return -1;
 
   imap_cmd_start (idata, buf);
 
   do 
   {
-    if (mutt_socket_readln (buf, sizeof (buf), conn) < 0)
-      return -1;
+    if ((rc = imap_cmd_step (idata)) != IMAP_CMD_CONTINUE)
+      break;
 
-    if (buf[0] == '*') 
+    s = imap_next_word (idata->buf);
+    if (mutt_strncasecmp ("STATUS", s, 6) == 0)
     {
-      s = imap_next_word (buf);
-      if (mutt_strncasecmp ("STATUS", s, 6) == 0)
+      s = imap_next_word (s);
+      /* The mailbox name may or may not be quoted here. We could try to 
+       * munge the server response and compare with quoted (or vise versa)
+       * but it is probably more efficient to just strncmp against both. */
+      if (mutt_strncmp (mbox_unquoted, s, mutt_strlen (mbox_unquoted)) == 0
+	  || mutt_strncmp (mbox, s, mutt_strlen (mbox)) == 0)
       {
 	s = imap_next_word (s);
-	/* The mailbox name may or may not be quoted here. We could try to 
-	 * munge the server response and compare with quoted (or vise versa)
-	 * but it is probably more efficient to just strncmp against both. */
-	if (mutt_strncmp (mbox_unquoted, s, mutt_strlen (mbox_unquoted)) == 0
-	    || mutt_strncmp (mbox, s, mutt_strlen (mbox)) == 0)
+	s = imap_next_word (s);
+	if (isdigit (*s))
 	{
-	  s = imap_next_word (s);
-	  s = imap_next_word (s);
-	  if (isdigit (*s))
+	  if (*s != '0')
 	  {
-	    if (*s != '0')
-	    {
-	      dprint (1, (debugfile, "Mail in %s\n", path));
-	      msgcount = atoi(s);
-	    }
+	    msgcount = atoi(s);
+	    dprint (2, (debugfile, "%d new messages in %s\n", msgcount, path));
 	  }
 	}
-	else
-	  dprint (1, (debugfile, "imap_mailbox_check: STATUS response doesn't match requested mailbox.\n"));
       }
       else
-      {
-	if (conn->data && 
-	    imap_handle_untagged (idata, buf) != 0)
-	  return -1;
-      }
+	dprint (1, (debugfile, "imap_mailbox_check: STATUS response doesn't match requested mailbox.\n"));
     }
   }
-  while ((mutt_strncmp (buf, idata->seq, SEQLEN) != 0));
-
-  imap_cmd_finish (idata);
+  while (rc == IMAP_CMD_CONTINUE);
 
   return msgcount;
 }
 
-int imap_parse_list_response(IMAP_DATA* idata, char *buf, int buflen,
-  char **name, int *noselect, int *noinferiors, char *delim)
+/* all this listing/browsing is a mess. I don't like that name is a pointer
+ *   into idata->buf (used to be a pointer into the passed in buffer, just
+ *   as bad), nor do I like the fact that the fetch is done here. This
+ *   code can't possibly handle non-LIST untagged responses properly.
+ *   FIXME. ?! */
+int imap_parse_list_response(IMAP_DATA* idata, char **name, int *noselect,
+  int *noinferiors, char *delim)
 {
   char *s;
   long bytes;
+  int rc;
 
   *name = NULL;
 
-  if (mutt_socket_readln (buf, buflen, idata->conn) < 0)
+  rc = imap_cmd_step (idata);
+  if (rc == IMAP_CMD_DONE)
+    return 0;
+  if (rc != IMAP_CMD_CONTINUE)
     return -1;
 
-  if (buf[0] == '*')
+  s = imap_next_word (idata->buf);
+  if ((mutt_strncasecmp ("LIST", s, 4) == 0) ||
+      (mutt_strncasecmp ("LSUB", s, 4) == 0))
   {
-    s = imap_next_word (buf);
-    if ((mutt_strncasecmp ("LIST", s, 4) == 0) ||
-	(mutt_strncasecmp ("LSUB", s, 4) == 0))
-    {
-      *noselect = 0;
-      *noinferiors = 0;
+    *noselect = 0;
+    *noinferiors = 0;
       
-      s = imap_next_word (s); /* flags */
-      if (*s == '(')
+    s = imap_next_word (s); /* flags */
+    if (*s == '(')
+    {
+      char *ep;
+
+      s++;
+      ep = s;
+      while (*ep && *ep != ')') ep++;
+      do
       {
-	char *ep;
-	
-	s++;
-	ep = s;
-	while (*ep && *ep != ')') ep++;
-	do {
-	  if (!strncasecmp (s, "\\NoSelect", 9))
-	    *noselect = 1;
-	  if (!strncasecmp (s, "\\NoInferiors", 12))
-	    *noinferiors = 1;
-	  if (*s != ')')
-	    s++;
-	  while (*s && *s != '\\' && *s != ')') s++;
-	} while (s != ep);
-      }
-      else
-	return (0);
-      s = imap_next_word (s); /* delim */
-      /* Reset the delimiter, this can change */
-      if (strncmp (s, "NIL", 3))
-      {
-	if (s && s[0] == '\"' && s[1] && s[2] == '\"')
-	  *delim = s[1];
-	else if (s && s[0] == '\"' && s[1] && s[1] == '\\' && s[2] && s[3] == '\"')
-	  *delim = s[2];
-      }
-      s = imap_next_word (s); /* name */
-      if (s && *s == '{')	/* Literal */
-      { 
-	int len;
-	
-	if (imap_get_literal_count(buf, &bytes) < 0)
-	  return -1;
-	len = mutt_socket_readln (buf, buflen, idata->conn);
-	if (len < 0)
-	  return -1;
-	*name = buf;
-      }
-      else
-	*name = s;
+	if (!strncasecmp (s, "\\NoSelect", 9))
+	  *noselect = 1;
+	if (!strncasecmp (s, "\\NoInferiors", 12))
+	  *noinferiors = 1;
+	if (*s != ')')
+	  s++;
+	while (*s && *s != '\\' && *s != ')') s++;
+      } while (s != ep);
     }
     else
+      return 0;
+    s = imap_next_word (s); /* delim */
+    /* Reset the delimiter, this can change */
+    if (strncmp (s, "NIL", 3))
     {
-      if (imap_handle_untagged (idata, buf) != 0)
-	return -1;
+      if (s && s[0] == '\"' && s[1] && s[2] == '\"')
+	*delim = s[1];
+      else if (s && s[0] == '\"' && s[1] && s[1] == '\\' && s[2] && s[3] == '\"')
+	*delim = s[2];
     }
+    s = imap_next_word (s); /* name */
+    if (s && *s == '{')	/* Literal */
+    { 
+      if (imap_get_literal_count(idata->buf, &bytes) < 0)
+	return -1;
+      if (imap_cmd_step (idata) != IMAP_CMD_CONTINUE)
+	return -1;
+      *name = idata->buf;
+    }
+    else
+      *name = s;
   }
 
   return 0;
@@ -1434,7 +1219,7 @@ int imap_subscribe (char *path, int subscribe)
     return -1;
 
   if (!(idata = imap_conn_find (&(mx.account), 0)))
-    return -1;
+    goto fail;
   
   conn = idata->conn;
 
@@ -1448,10 +1233,15 @@ int imap_subscribe (char *path, int subscribe)
   snprintf (buf, sizeof (buf), "%s %s", subscribe ? "SUBSCRIBE" :
     "UNSUBSCRIBE", mbox);
 
-  if (imap_exec (buf, sizeof (buf), idata, buf, 0) < 0)
-    return -1;
+  if (imap_exec (idata, buf, 0) < 0)
+    goto fail;
 
+  FREE (&mx.mbox);
   return 0;
+
+ fail:
+  FREE (&mx.mbox);
+  return -1;
 }
 
 /* imap_complete: given a partial IMAP folder path, return a string which
@@ -1479,12 +1269,12 @@ int imap_complete(char* dest, size_t dlen, char* path) {
 
   /* don't open a new socket just for completion */
   if (!(idata = imap_conn_find (&(mx.account), M_IMAP_CONN_NONEW)))
-    return -1;
+    goto fail;
   conn = idata->conn;
 
   /* reformat path for IMAP list, and append wildcard */
   /* don't use INBOX in place of "" */
-  if (mx.mbox[0])
+  if (mx.mbox && mx.mbox[0])
     imap_fix_path (idata, mx.mbox, list, sizeof(list));
   else
     list[0] = '\0';
@@ -1496,11 +1286,11 @@ int imap_complete(char* dest, size_t dlen, char* path) {
   imap_cmd_start (idata, buf);
 
   /* and see what the results are */
-  strfcpy (completion, mx.mbox, sizeof(completion));
+  strfcpy (completion, NONULL(mx.mbox), sizeof(completion));
   do
   {
-    if (imap_parse_list_response(idata, buf, sizeof(buf), &list_word,
-        &noselect, &noinferiors, &delim))
+    if (imap_parse_list_response(idata, &list_word, &noselect, &noinferiors,
+        &delim))
       break;
 
     if (list_word)
@@ -1535,7 +1325,7 @@ int imap_complete(char* dest, size_t dlen, char* path) {
       completions++;
     }
   }
-  while (mutt_strncmp(idata->seq, buf, SEQLEN));
+  while (mutt_strncmp(idata->seq, idata->buf, SEQLEN));
 
   if (completions)
   {
@@ -1543,8 +1333,11 @@ int imap_complete(char* dest, size_t dlen, char* path) {
     imap_qualify_path (dest, dlen, &mx, completion, NULL);
     mutt_pretty_mailbox (dest);
 
+    FREE (&mx.mbox);
     return 0;
   }
 
+ fail:
+  FREE (&mx.mbox);
   return -1;
 }
