@@ -32,13 +32,20 @@
 #define IMAP_LOG_LTRL 4
 #define IMAP_LOG_PASS 5
 
+/* IMAP command responses */
+#define IMAP_CMD_DONE     (0)
+#define IMAP_CMD_FAIL     (-1)
+#define IMAP_CMD_NO       (-2)
+#define IMAP_CMD_CONTINUE (1)
+#define IMAP_CMD_RESPOND  (2)
+
 /* number of entries in the hash table */
 #define IMAP_CACHE_LEN 10
 
 #define SEQLEN 5
 
 #define IMAP_REOPEN_ALLOW    (1<<0)
-#define IMAP_REOPEN_PENDING  (1<<1)
+#define IMAP_EXPUNGE_PENDING (1<<1)
 #define IMAP_NEWMAIL_PENDING (1<<2)
 
 /* imap_exec flags (see imap_exec) */
@@ -48,7 +55,6 @@
 enum
 {
   IMAP_FATAL = 1,
-  IMAP_NEW_MAIL,
   IMAP_BYE,
   IMAP_REOPENED
 };
@@ -86,7 +92,7 @@ enum
   RIGHTSMAX
 };
 
-/* Capabilities */
+/* Capabilities we are interested in */
 enum
 {
   IMAP4 = 0,
@@ -95,21 +101,10 @@ enum
   ACL,				/* RFC 2086: IMAP4 ACL extension */
   NAMESPACE,                   	/* RFC 2342: IMAP4 Namespace */
   ACRAM_MD5,			/* RFC 2195: CRAM-MD5 authentication */
-  AKERBEROS_V4,			/* AUTH=KERBEROS_V4 */
   AGSSAPI,			/* RFC 1731: GSSAPI authentication */
-  /* From here down, we don't care */
-  ALOGIN,			/* AUTH=LOGIN */
-  AUTH_LOGIN,			/* AUTH-LOGIN */
-  APLAIN,			/* AUTH=PLAIN */
-  ASKEY,			/* AUTH=SKEY */
-  IDLE,				/* RFC 2177: IMAP4 IDLE command */
-  LOGIN_REFERRALS,		/* LOGIN-REFERRALS */
-  MAILBOX_REFERRALS,		/* MAILBOX-REFERRALS */
-  SCAN,
-  SORT,
-  TORDEREDSUBJECT,		/* THREAD=ORDEREDSUBJECT */
-  UIDPLUS,			/* RFC 2859: IMAP4 UIDPLUS extension */
   AUTH_ANON,			/* AUTH=ANONYMOUS */
+  STARTTLS,			/* RFC 2595: STARTTLS */
+  LOGINDISABLED,		/*           LOGINDISABLED */
 
   CAPMAX
 };
@@ -143,8 +138,11 @@ typedef struct
   CONNECTION *conn;
   unsigned char state;
   unsigned char status;
-  unsigned char check_status;
   unsigned char capabilities[(CAPMAX + 7)/8];
+  char seq[SEQLEN+1];
+  /* command input buffer */
+  char* buf;
+  unsigned int blen;
   /* let me explain capstr: SASL needs the capability string (not bits).
    * we have 3 options:
    *   1. rerun CAPABILITY inside SASL function.
@@ -154,12 +152,12 @@ typedef struct
    * tracking all possible capabilities. bah. (1) I don't like because
    * it's just no fun to get the same information twice */
   char* capstr;
-  char seq[SEQLEN+1];
 
   /* The following data is all specific to the currently SELECTED mbox */
   char delim;
   CONTEXT *ctx;
   char *mailbox;
+  unsigned short check_status;
   unsigned char reopen;
   unsigned char rights[(RIGHTSMAX + 7)/8];
   unsigned int newMailCount;
@@ -175,33 +173,31 @@ typedef struct
 
 /* -- private IMAP functions -- */
 /* imap.c */
+int imap_create_mailbox (IMAP_DATA* idata, char* mailbox);
 int imap_make_msg_set (IMAP_DATA* idata, char* buf, size_t buflen, int flag,
   int changed);
 int imap_open_connection (IMAP_DATA* idata);
 IMAP_DATA* imap_conn_find (const ACCOUNT* account, int flags);
 time_t imap_parse_date (char* s);
-int imap_parse_list_response(IMAP_DATA* idata, char* buf, int buflen,
-  char** name, int* noselect, int* noinferiors, char* delim);
+int imap_parse_list_response(IMAP_DATA* idata, char** name, int* noselect,
+  int* noinferiors, char* delim);
 int imap_read_literal (FILE* fp, IMAP_DATA* idata, long bytes);
 void imap_expunge_mailbox (IMAP_DATA* idata);
-int imap_reopen_mailbox (IMAP_DATA* idata);
 void imap_logout (IMAP_DATA* idata);
 
 /* auth.c */
 int imap_authenticate (IMAP_DATA* idata);
 
 /* command.c */
-void imap_cmd_start (IMAP_DATA* idata, const char* cmd);
-void imap_cmd_finish (IMAP_DATA* idata);
+int imap_cmd_start (IMAP_DATA* idata, const char* cmd);
+int imap_cmd_step (IMAP_DATA* idata);
 int imap_code (const char* s);
-int imap_exec (char* buf, size_t buflen, IMAP_DATA* idata, const char* cmd,
-  int flags);
-int imap_handle_untagged (IMAP_DATA* idata, char* s);
+int imap_exec (IMAP_DATA* idata, const char* cmd, int flags);
 
 /* message.c */
 void imap_add_keywords (char* s, HEADER* keywords, LIST* mailbox_flags);
 void imap_free_header_data (void** data);
-int imap_read_headers (CONTEXT* ctx, int msgbegin, int msgend);
+int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend);
 
 /* util.c */
 int imap_continue (const char* msg, const char* resp);
