@@ -32,8 +32,7 @@
 
 
 #ifdef USE_SSL
-#include "imap.h"
-#include "imap_ssl.h"
+#include "mutt_ssl.h"
 #endif
 
 
@@ -1767,6 +1766,8 @@ void mutt_init (int skip_sys_rc, LIST *commands)
 
   if ((p = getenv ("MAIL")))
     Spoolfile = safe_strdup (p);
+  else if ((p = getenv ("MAILDIR")))
+    Spoolfile = safe_strdup (p);
   else
   {
 #ifdef HOMESPOOL
@@ -1836,6 +1837,12 @@ void mutt_init (int skip_sys_rc, LIST *commands)
     set_option (OPTLOCALES);
 #endif
 
+#ifdef HAVE_GETSID
+  /* Unset suspend by default if we're the session leader */
+  if (getsid(0) == getpid())
+    unset_option (OPTSUSPEND);
+#endif
+
   mutt_init_history ();
 
   
