@@ -22,6 +22,7 @@
 #include "mime.h"
 #include "mailbox.h"
 #include "mx.h"
+#include "url.h"
 
 #ifdef HAVE_PGP
 #include "pgp.h"
@@ -663,6 +664,17 @@ void mutt_pretty_mailbox (char *s)
   char *p = s, *q = s;
   size_t len;
 
+  /* if s is an url, only collapse path component */
+  if (url_check_scheme (s) != U_UNKNOWN)
+  {
+    p = strchr(s, ':')+1;
+    if (!strncmp (p, "//", 2))
+      q = strchr (p+2, '/');
+    if (!q)
+      q = strchr (p, '\0');
+    p = q;
+  }
+  
   /* first attempt to collapse the pathname */
   while (*p)
   {
@@ -681,16 +693,22 @@ void mutt_pretty_mailbox (char *s)
   }
   *q = 0;
 
-  if (mutt_strncmp (s, Maildir, (len = mutt_strlen (Maildir))) == 0 && s[len] == '/')
+  if (Maildir && mutt_strncmp (s, Maildir, (len = mutt_strlen (Maildir))) == 0)
   {
+    /* chop off delimiters after Maildir. '.' is a hack because IMAP can
+     * use arbitrary delimiters, but we may not have a connection. Could
+     * call an IMAP delimiter getting function here later, otherwise
+     * default to this. */
+    if (s[len] != '/' && s[len] != '.')
+      len--;
     *s++ = '=';
-    strcpy (s, s + len);
+    memmove (s, s + len, mutt_strlen (s + len) + 1);
   }
   else if (mutt_strncmp (s, Homedir, (len = mutt_strlen (Homedir))) == 0 &&
 	   s[len] == '/')
   {
     *s++ = '~';
-    strcpy (s, s + len - 1);
+    memmove (s, s + len - 1, mutt_strlen (s + len - 1) + 1);
   }
 }
 
